"""Custom exceptions for session-hub-core."""


class SessionDBError(Exception):
    """Base exception for session-hub-core."""


class ConnectionError(SessionDBError):
    """Cannot connect to PostgreSQL."""


class SyncError(SessionDBError):
    """Error during sync operation."""


class QueryError(SessionDBError):
    """Error during query operation."""


class SessionNotFoundError(SessionDBError):
    """Session not found in the database."""
